require 'resque/errors'
module Actionable
  class Job

    attr_reader :actionable, :target, :payload

    def self.logger
      defined?(Rails) ? Rails.logger : Resque.logger
    end

    def logger
      self.class.logger
    end

    def self.queue
      @queue ||= 'actionable'
    end

    def self.before_enqueue(id)
      actionable = Actionable::Action.find(id)
      actionable.update_attributes(status: :enqueued)
      true
    end

    def self.on_failure(err,id)
      begin
        actionable = Actionable::Action.find(id)
        actionable.update_attributes({
          status: :failed,
          exception_message: err.message,
          exception_backtrace: err.backtrace
        })
        job = new(actionable.target,actionable.payload,actionable)
        job.on_failure(err) if job.respond_to?(:on_failure)
      rescue => e
        puts "Error in 'on_failure' handler: #{e.message}"
        logger.error("Error in 'on_failure' handler: #{e.message}")
        raise
      end
      raise err
    end

    def self.perform(id)
      actionable = Actionable::Action.find(id)
      actionable.update_attributes(status: :working)
      job = new(actionable.target,actionable.payload,actionable)
      job.perform
      actionable.update_attributes(status: :complete)
      job.on_success if job.respond_to?(:on_success)
    end

    def initialize(target,payload={},actionable=nil)
      @target = target
      @actionable = actionable
      @payload = HashWithIndifferentAccess.new(payload)
    end

  end
end
