require 'active_model'

module Actionable
  module MemoryStore
    class Action

      include ActiveModel::Model
      attr_accessor :id, :execution_time, :payload, :job_class_name 
      attr_accessor :target, :target_id, :target_class_name, :status

      def self.create(data)
        this = new(data)
        this.status = :new
        this
      end

      def self.build(data)
        this = new(data)
        this.status = :new
        this
      end

      def target
        @target ||= get_target_from_definition
      end

      def save
        Actionable::MemoryStore.insert(for_save)
      end

      def enqueue
        update_attributes(status: :enqueued)
        Resque.enqueue(job_class,id.to_s)
      end

      def cancel
        update_attributes(status: :canceled)
      end

      alias :cancel! :cancel

      def reschedule_for(execution_time)
        update_attributes(execution_time: execution_time)
      end

      protected

      def set_target_definition
        target_class = target.class.to_s
        target_id = target.id
      end

      def get_target_from_definition
        return nil unless target_class.present? && target_id.present?
        target_class.find(target_id)
      end

      def job_class
        job_class_name.constantize if job_class_name.present?
      end

      def target_class
        target_class_name.constantize if target_class_name.present?
      end

      def update_attributes(hash)
        hash.each_pair do |key,value|
          instance_variable_set(:"@#{key}",value)
        end
      end

      private

      def for_save
        if @target.present? && (@target_id.nil? || @target_class.nil?)
          set_target_definition
        end
        json = as_json
        json.delete(:target)
        json
      end

    end
  end
end

