require 'mongoid'

module Actionable
  module MongoidStore
    class Action < MemoryStore::Action

      include Mongoid::Document

      store_in(collection: 'actionables')

      after_initialize :flag_late

      field :execution_time,    type: Time
      field :payload,           type: Hash
      field :job_class_name,    type: String
      field :status,            type: Symbol,     default: :new
      field :target_id,         type: BSON::ObjectId
      field :target_class_name, type: String
      field :exception_message,   type: String
      field :exception_backtrace, type: Array

      scope :scheduled_for_between, ->(from,to){
        between(execution_time:[from,to])
      }
      scope :scheduled_for_before, ->(to){ where(:execution_time.lt => to) }

      scope :with_job_class, ->(klass) { where(job_class_name: klass.to_s.classify)}

      scope :scheduled_or_late, -> { where(:status.in => [:scheduled,:late]) }

      [:scheduled,:enqueued,:working,:complete,:canceled,:failed,:overdue].each do |status|
        scope status, -> { where(status: status) }
        define_method(:"#{status}?") { self.status == status }
      end

      def self.to_do
        scheduled_or_late.scheduled_for_before(to_do_time)
      end

      def self.to_do_time
        DateTime.now.utc
      end

      def late?
        execution_time.present? && execution_time < 1.minute.from_now && scheduled?
      end

      def flag_late
        update_attribute(:status,:late) if late?
      end

    end
  end
end

