require 'rufus/scheduler'

module Actionable
  class Recurrence

      include Mongoid::Document
      include Actionable::Target

      store_in(collection: 'recurring_actionables')

      field :recurrence_string
      field :job_class,         type: String
      field :payload,           type: Hash,   default: {}

      after_create :schedule_next

      validate :valid_recurrence_string
      validate :valid_job_class

      def schedule_next
        schedule_actionable(next_run_time,job_class,payload)
      end

      def next_scheduled?
        Actionable::Action.where(:status.in => [:scheduled,:late,:enqueued,:working], target_id: id).exists?
      end

      def ensure_scheduled
        schedule_next unless next_scheduled?
      end

      private

      def next_run_time
        Rufus::CronLine.new(recurrence_string).next_time.to_datetime.utc
      end

      def job_class
        unless read_attribute(:job_class).blank?
          read_attribute(:job_class).constantize 
        end
      end

      def valid_recurrence_string
        begin
          next_run_time
        rescue => e
          errors.add(:recurrence_string,e.message)
        end
      end

      def valid_job_class
        begin
          job_class
        rescue => e
          errors.add(:job_class,e.message)
        end
      end

  end
end