require 'active_support'

module Actionable 
  module Target
  
    extend ActiveSupport::Concern

    attr_accessor :actionable_ids
    attr_accessor :actionable_errors

    def actionables
      @actionables ||= Actionable::Action.where(target_id: id)
    end

    def actionable_errors
      @actionable_errors
    end

    def schedule_actionable(execution_time,job_class_name,payload={})
      
      # force memoized values to refresh
      @actionables = nil
      @actionable_errors = nil

      # set model values
      actionable = Actionable::Action.build(
        target_id:id,
        target_class_name: self.class.to_s,
        payload: payload,
        execution_time: execution_time.to_datetime.utc,
        job_class_name: job_class_name
      )

      # yield the model to a block for 
      # situation-specific adjustments
      yield(actionable) if block_given?

      # set status to saved now that 
      # initialization is complete
      actionable.status = :scheduled

      # save the new model record
      unless actionable.save

        # keep error values around for inspection
        @actionable_errors = actionable.errors
        false
      end
      true

    end

  end
end